$LOAD_PATH.unshift(__dir__)

require "joya/version"
require "joya/gem"
require 'open-uri'
require 'nokogiri'


%w[INT TERM].each do |signal|
  trap(signal) do
    puts "\b"*3+"Download interrupted"
    break
  end
end

class String # :nodoc:
  # check if a gem is installed
  def gem_installed?
    gem_path=ENV['GEM_PATH'].split(":").first
    !Dir.glob([gem_path, 'gems', self + Joya::GEM_VERSION_PATTERN].join('/')).empty?
  rescue NoMethodError
  end
end

# the Joya module
module Joya
  class Error < StandardError; end # :nodoc:

  # xpath pattern to scrap gem names of a rubygems user
  GEM_NAME_XPATH="//a[@class='gems__gem__name']//text()[not(parent::span)]"
  GEM_VERSION_PATTERN='-[0-9]*\.[0-9]*\.[0-9]*'

  @gems=[]
  @author=''

  class << self
    # gems author
    attr_accessor :author

    # author's gems
    attr_accessor :gems

    def main(args) # :nodoc:
      help if(args.include?("-h") or args.include?("--help") or args.empty?)
      version if (args.include?("-v") or args.include?("--version"))

      # take the first argument starting with '\w' as the author
      @author = args.find{|arg| arg.match(/^\w/)}
      help if author.nil?

      fetch_gem_names
      print_and_quit if (args.include?("-p") or args.include?("--print"))
      make_dir
      unpack_all
    end

    # print user gems and exit
    def print_and_quit
      puts "Gems of #{author}:"
      puts gems
      exit
    end

    # install missing gems and unpack them in +base_dir+
    def unpack_all
      Dir.chdir base_dir
      total=gems.size
      digits=total.to_s.size
      puts "#{total} gems found"
      puts "Downloading into #{base_dir}/"
      interrupted = false
      gems.each_with_index do |jem,i|
        printf("[%#{digits}d/%#{digits}d] Unpacking %s\n", i+1, total, jem)
        Joya::Gem.install(jem) unless jem.gem_installed?
        Joya::Gem.unpack(jem)
      end
    rescue LocalJumpError
    ensure
      Dir.chdir '..'
    end

    def version # :nodoc:
      puts "joya #{Joya::VERSION}"
      exit
    end

    def help # :nodoc:
      puts <<~str
      Usage:
        joya <author> [options]

      Options:
        -h, --help         print this help message
        -p, --print        print user gems without downloading
        -v, --version      print joya version
      str
      exit
    end

    def url # :nodoc:
      "https://rubygems.org/profiles/#{author}"
    end

    # fetch gem names from a given rubygems author
    def fetch_gem_names
      site=Nokogiri::HTML(open_rubygems)
      @gems=site.xpath(GEM_NAME_XPATH).text.gsub(/\s+/," ").split
    end

    # open the rubygems url
    def open_rubygems
      URI.open(url)
    rescue OpenURI::HTTPError
      puts "HTTPError: Could not find rubygems of #{author}"
      exit
    end

    # directory where gems will be unpacked
    def base_dir
      "gems-#{author}"
    end
    alias_method :directory, :base_dir

    # makes +base_dir+
    def make_dir
      FileUtils.mkdir base_dir
    rescue Errno::EEXIST
    end
  end
end
